@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2018 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2019 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -165,7 +165,7 @@ NOEXPORT void cron_dh_param(void) {
     SERVICE_OPTIONS *opt;
     DH *dh;
 
-    if(!dh_needed)
+    if(!dh_temp_params || !service_options.next)
         return;
 
     s_log(LOG_NOTICE, "Updating DH parameters");
@@ -190,15 +190,17 @@ NOEXPORT void cron_dh_param(void) {
 #endif
 
     /* update global dh_params for future configuration reloads */
-    stunnel_write_lock(&stunnel_locks[LOCK_DH]);
+    CRYPTO_THREAD_write_lock(stunnel_locks[LOCK_DH]);
     DH_free(dh_params);
     dh_params=dh;
-    stunnel_write_unlock(&stunnel_locks[LOCK_DH]);
+    CRYPTO_THREAD_unlock(stunnel_locks[LOCK_DH]);
 
     /* set for all sections that require it */
+    CRYPTO_THREAD_read_lock(stunnel_locks[LOCK_SECTIONS]);
     for(opt=service_options.next; opt; opt=opt->next)
-        if(opt->option.dh_needed)
+        if(opt->option.dh_temp_params)
             SSL_CTX_set_tmp_dh(opt->ctx, dh);
+    CRYPTO_THREAD_unlock(stunnel_locks[LOCK_SECTIONS]);
     s_log(LOG_NOTICE, "DH parameters updated");
 }
 #endif /* OPENSSL_NO_DH */
