@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2018 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2019 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -166,7 +166,7 @@ typedef struct servername_list_struct SERVERNAME_LIST;/* forward declaration */
 typedef struct psk_keys_struct {
     char *identity;
     unsigned char *key_val;
-    size_t key_len;
+    unsigned key_len;
     struct psk_keys_struct *next;
 } PSK_KEYS;
 typedef struct psk_table_struct {
@@ -216,6 +216,9 @@ typedef struct service_options_struct {
 
         /* service-specific data for ctx.c */
     char *cipher_list;
+#ifndef OPENSSL_NO_TLS1_3
+    char *ciphersuites;
+#endif /* TLS 1.3 */
     char *cert;                                             /* cert filename */
     char *key;                               /* pem (priv key/cert) filename */
     long session_size, session_timeout;
@@ -223,7 +226,11 @@ typedef struct service_options_struct {
 #if OPENSSL_VERSION_NUMBER>=0x009080dfL
     long unsigned ssl_options_clear;
 #endif /* OpenSSL 0.9.8m or later */
+#if OPENSSL_VERSION_NUMBER>=0x10100000L
+    int min_proto_version, max_proto_version;
+#else /* OPENSSL_VERSION_NUMBER<0x10100000L */
     SSL_METHOD *client_method, *server_method;
+#endif /* OPENSSL_VERSION_NUMBER<0x10100000L */
     SOCKADDR_UNION sessiond_addr;
 #ifndef OPENSSL_NO_TLSEXT
     char *sni;
@@ -235,7 +242,7 @@ typedef struct service_options_struct {
     PSK_TABLE psk_sorted;
 #endif /* !defined(OPENSSL_NO_PSK) */
 #ifndef OPENSSL_NO_ECDH
-    int curve;
+    char *curves;
 #endif /* !defined(OPENSSL_NO_ECDH) */
 #ifndef OPENSSL_NO_ENGINE
     ENGINE *engine;                        /* engine to read the private key */
@@ -305,8 +312,11 @@ typedef struct service_options_struct {
         unsigned nonce:1;               /* send and verify OCSP nonce */
 #endif /* !defined(OPENSSL_NO_OCSP) */
 #ifndef OPENSSL_NO_DH
-        unsigned dh_needed:1;
+        unsigned dh_temp_params:1;
 #endif /* OPENSSL_NO_DH */
+#ifndef USE_WIN32
+        unsigned log_stderr:1;          /* a copy of the global switch */
+#endif /* USE_WIN32 */
     } option;
 } SERVICE_OPTIONS;
 
@@ -411,13 +421,16 @@ typedef struct {
     FD *ssl_rfd, *ssl_wfd;                 /* read and write TLS descriptors */
     uint64_t sock_bytes, ssl_bytes;       /* bytes written to socket and TLS */
     s_poll_set *fds;                                     /* file descriptors */
+    struct {
+        unsigned psk:1;                            /* PSK identity was found */
+    } flag;
 } CLI;
 
 /**************************************** prototypes for stunnel.c */
 
 #ifndef USE_FORK
-extern long max_clients;
-extern volatile long num_clients;
+extern int max_clients;
+extern int num_clients;
 #endif
 
 void main_init(void);
@@ -429,13 +442,13 @@ void unbind_ports(void);
 int bind_ports(void);
 void signal_post(uint8_t);
 #if !defined(USE_WIN32) && !defined(USE_OS2)
-void child_status(void);  /* dead libwrap or 'exec' process detected */
+void pid_status_hang(const char *);
 #endif
 void stunnel_info(int);
 
 /**************************************** prototypes for options.c */
 
-extern char configuration_file[PATH_MAX];
+extern char *configuration_file;
 extern unsigned number_of_sections;
 
 int options_cmdline(char *, char *);
@@ -460,12 +473,11 @@ void set_nonblock(SOCKET, unsigned long);
 
 /**************************************** prototypes for log.c */
 
-#if !defined(USE_WIN32) && !defined(__vms)
-void syslog_open(void);
-void syslog_close(void);
-#endif
-int log_open(void);
-void log_close(void);
+#define SINK_SYSLOG 1
+#define SINK_OUTFILE 2
+
+int log_open(int);
+void log_close(int);
 void log_flush(LOG_MODE);
 void s_log(int, const char *, ...)
 #ifdef __GNUC__
@@ -480,6 +492,7 @@ void ioerror(const char *);
 void sockerror(const char *);
 void log_error(int, int, const char *);
 char *s_strerror(int);
+void bin2hexstring(const unsigned char *, size_t, char *, size_t);
 
 /**************************************** prototypes for pty.c */
 
@@ -507,7 +520,7 @@ extern SERVICE_OPTIONS *current_section;
 
 #ifndef OPENSSL_NO_DH
 extern DH *dh_params;
-extern int dh_needed;
+extern int dh_temp_params;
 #endif /* OPENSSL_NO_DH */
 
 int context_init(SERVICE_OPTIONS *);
@@ -515,6 +528,7 @@ int context_init(SERVICE_OPTIONS *);
 void psk_sort(PSK_TABLE *, PSK_KEYS *);
 PSK_KEYS *psk_find(const PSK_TABLE *, const char *);
 #endif /* !defined(OPENSSL_NO_PSK) */
+void print_session_id(SSL_SESSION *);
 void sslerror(char *);
 
 /**************************************** prototypes for verify.c */
@@ -631,7 +645,7 @@ const char *s_gai_strerror(int);
 #ifndef _WIN32_WCE
 typedef int (CALLBACK * GETADDRINFO) (const char *,
     const char *, const struct addrinfo *, struct addrinfo **);
-typedef void (CALLBACK * FREEADDRINFO) (struct addrinfo FAR *);
+typedef void (CALLBACK * FREEADDRINFO) (struct addrinfo *);
 typedef int (CALLBACK * GETNAMEINFO) (const struct sockaddr *, socklen_t,
     char *, size_t, char *, size_t, int);
 extern GETADDRINFO s_getaddrinfo;
@@ -652,6 +666,8 @@ int getnameinfo(const struct sockaddr *, socklen_t,
 #define USE_OS_THREADS
 #endif
 
+#if OPENSSL_VERSION_NUMBER<0x10100004L
+
 #ifdef USE_OS_THREADS
 
 struct CRYPTO_dynlock_value {
@@ -662,11 +678,20 @@ struct CRYPTO_dynlock_value {
     CRITICAL_SECTION critical_section;
 #endif
     const char *init_file, *read_lock_file, *write_lock_file,
-        *read_unlock_file, *write_unlock_file, *destroy_file;
-    int init_line, read_lock_line, write_lock_line,
-        read_unlock_line, write_unlock_line, destroy_line;
+        *unlock_file, *destroy_file;
+    int init_line, read_lock_line, write_lock_line, unlock_line, destroy_line;
 };
 
+typedef struct CRYPTO_dynlock_value CRYPTO_RWLOCK;
+
+#else /* USE_OS_THREADS */
+
+typedef void CRYPTO_RWLOCK;
+
+#endif /* USE_OS_THREADS */
+
+#endif /* OPENSSL_VERSION_NUMBER<0x10100004L */
+
 typedef enum {
     LOCK_SESSION, LOCK_ADDR,
     LOCK_CLIENTS, LOCK_SSL,                 /* client.c */
@@ -680,40 +705,25 @@ typedef enum {
 #ifndef OPENSSL_NO_DH
     LOCK_DH,                                /* ctx.c */
 #endif /* OPENSSL_NO_DH */
+#ifdef USE_WIN32
+    LOCK_WIN_LOG,                           /* ui_win_gui.c */
+#endif
+    LOCK_SECTIONS,                          /* traversing section list */
     STUNNEL_LOCKS                           /* number of locks */
 } LOCK_TYPE;
-extern struct CRYPTO_dynlock_value stunnel_locks[STUNNEL_LOCKS];
 
-void stunnel_rwlock_init_debug(struct CRYPTO_dynlock_value *, const char *, int);
-void stunnel_read_lock_debug(struct CRYPTO_dynlock_value *, const char *, int);
-void stunnel_write_lock_debug(struct CRYPTO_dynlock_value *, const char *, int);
-void stunnel_read_unlock_debug(struct CRYPTO_dynlock_value *, const char *, int);
-void stunnel_write_unlock_debug(struct CRYPTO_dynlock_value *, const char *, int);
-void stunnel_rwlock_destroy_debug(struct CRYPTO_dynlock_value *, const char *, int);
-
-#define stunnel_rwlock_init(x) stunnel_rwlock_init_debug((x),__FILE__,__LINE__)
-#define stunnel_read_lock(x) stunnel_read_lock_debug((x),__FILE__,__LINE__)
-#define stunnel_write_lock(x) stunnel_write_lock_debug((x),__FILE__,__LINE__)
-#define stunnel_read_unlock(x) stunnel_read_unlock_debug((x),__FILE__,__LINE__)
-#define stunnel_write_unlock(x) stunnel_write_unlock_debug((x),__FILE__,__LINE__)
-#define stunnel_rwlock_destroy(x) stunnel_rwlock_destroy_debug((x),__FILE__,__LINE__)
+extern CRYPTO_RWLOCK *stunnel_locks[STUNNEL_LOCKS];
 
 #if OPENSSL_VERSION_NUMBER<0x10100004L
-#define CRYPTO_atomic_add(addr,amount,result,type) \
-    *result = type ? CRYPTO_add(addr,amount,type) : (*addr+=amount)
+/* Emulate the OpenSSL 1.1 locking API for older OpenSSL versions */
+CRYPTO_RWLOCK *CRYPTO_THREAD_lock_new(void);
+int CRYPTO_THREAD_read_lock(CRYPTO_RWLOCK *);
+int CRYPTO_THREAD_write_lock(CRYPTO_RWLOCK *);
+int CRYPTO_THREAD_unlock(CRYPTO_RWLOCK *);
+void CRYPTO_THREAD_lock_free(CRYPTO_RWLOCK *);
+int CRYPTO_atomic_add(int *, int, int *, CRYPTO_RWLOCK *);
 #endif
 
-#else /* USE_OS_THREADS */
-
-#define stunnel_rwlock_init(x) {}
-#define stunnel_read_lock(x) {}
-#define stunnel_write_lock(x) {}
-#define stunnel_read_unlock(x) {}
-#define stunnel_write_unlock(x) {}
-#define stunnel_rwlock_destroy(x) {}
-
-#endif /* USE_OS_THREADS */
-
 int sthreads_init(void);
 unsigned long stunnel_process_id(void);
 unsigned long stunnel_thread_id(void);
@@ -737,7 +747,8 @@ long _beginthread(void (*)(void *), int, void *);
 void _endthread(void);
 #endif
 #ifdef DEBUG_STACK_SIZE
-void stack_info(int);
+void stack_info(size_t, int);
+void ignore_value(void *);
 #endif
 
 /**************************************** prototypes for file.c */
@@ -817,6 +828,8 @@ void str_free_debug(void *, const char *, int);
 #define str_free(a) str_free_debug((a), __FILE__, __LINE__), (a)=NULL
 #define str_free_expression(a) str_free_debug((a), __FILE__, __LINE__)
 
+void leak_table_utilization(void);
+
 int safe_memcmp(const void *, const void *, size_t);
 
 /**************************************** prototypes for ui_*.c */
