@@ -1,6 +1,6 @@
 /*
  *   stunnel       TLS offloading and load-balancing proxy
- *   Copyright (C) 1998-2018 Michal Trojnara <Michal.Trojnara@stunnel.org>
+ *   Copyright (C) 1998-2019 Michal Trojnara <Michal.Trojnara@stunnel.org>
  *
  *   This program is free software; you can redistribute it and/or modify it
  *   under the terms of the GNU General Public License as published by the
@@ -106,6 +106,11 @@ void resolver_init() {
 }
 
 #if defined(USE_WIN32) && !defined(_WIN32_WCE)
+#ifdef __GNUC__
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wpragmas"
+#pragma GCC diagnostic ignored "-Wcast-function-type"
+#endif /* __GNUC__ */
 NOEXPORT int get_ipv6(LPTSTR file) {
     HINSTANCE handle;
 
@@ -124,6 +129,9 @@ NOEXPORT int get_ipv6(LPTSTR file) {
     }
     return 1; /* IPv6 detected -> OK */
 }
+#ifdef __GNUC__
+#pragma GCC diagnostic pop
+#endif /* __GNUC__ */
 #endif
 
 /**************************************** stunnel resolver API */
@@ -350,16 +358,20 @@ char *s_ntop(SOCKADDR_UNION *addr, socklen_t addrlen) {
 }
 
 socklen_t addr_len(const SOCKADDR_UNION *addr) {
-    if(addr->sa.sa_family==AF_INET)
+    switch(addr->sa.sa_family) {
+    case AF_UNSPEC: /* 0 */
+        return 0;
+    case AF_INET: /* 2 (almost universally) */
         return sizeof(struct sockaddr_in);
 #ifdef USE_IPv6
-    if(addr->sa.sa_family==AF_INET6)
+    case AF_INET6:
         return sizeof(struct sockaddr_in6);
 #endif
 #ifdef HAVE_STRUCT_SOCKADDR_UN
-    if(addr->sa.sa_family==AF_UNIX)
+    case AF_UNIX:
         return sizeof(struct sockaddr_un);
 #endif
+    }
     s_log(LOG_ERR, "INTERNAL ERROR: Unknown sa_family: %d",
         addr->sa.sa_family);
     return sizeof(SOCKADDR_UNION);
@@ -431,7 +443,7 @@ NOEXPORT int getaddrinfo(const char *node, const char *service,
     /* not numerical: need to call resolver library */
     *res=NULL;
     ai=NULL;
-    stunnel_write_lock(&stunnel_locks[LOCK_INET]);
+    CRYPTO_THREAD_write_lock(stunnel_locks[LOCK_INET]);
 #ifdef HAVE_GETHOSTBYNAME2
     h=gethostbyname2(node, AF_INET6);
     if(h) /* some IPv6 addresses found */
@@ -449,7 +461,7 @@ NOEXPORT int getaddrinfo(const char *node, const char *service,
 #ifdef HAVE_ENDHOSTENT
     endhostent();
 #endif
-    stunnel_write_unlock(&stunnel_locks[LOCK_INET]);
+    CRYPTO_THREAD_unlock(stunnel_locks[LOCK_INET]);
     if(retval) { /* error: free allocated memory */
         freeaddrinfo(*res);
         *res=NULL;
@@ -584,10 +596,11 @@ int getnameinfo(const struct sockaddr *sa, socklen_t salen,
                 (void *)&((struct sockaddr_in *)sa)->sin_addr,
             host, hostlen);
 #else /* USE_IPv6 */
-        stunnel_write_lock(&stunnel_locks[LOCK_INET]); /* inet_ntoa is not mt-safe */
+        /* inet_ntoa is not mt-safe */
+        CRYPTO_THREAD_write_lock(stunnel_locks[LOCK_INET]);
         strncpy(host, inet_ntoa(((struct sockaddr_in *)sa)->sin_addr),
             hostlen);
-        stunnel_write_unlock(&stunnel_locks[LOCK_INET]);
+        CRYPTO_THREAD_unlock(stunnel_locks[LOCK_INET]);
         host[hostlen-1]='\0';
 #endif /* USE_IPv6 */
     }
